//
//  Photo+Flickr.m
//  Photomania
//
//  Created by Sameh Fakhouri on 12/5/14.
//  Copyright (c) 2014 CUNY Lehman College. All rights reserved.
//

#import "Photo+Flickr.h"
#import "FlickrFetcher.h"
#import "Photographer+Create.h"

@implementation Photo (Flickr)

//
// given a photo dictionary from Flickr, this method will give us
// back a Photo object in the database, either by finding an
// existing photo with the same id or by creating an new one.
//
// in case of an error, this method returns nil.
//

+ (Photo *)photoWithFlickrInfo:(NSDictionary *)photoDictionary
        inManagedObjectContext:(NSManagedObjectContext *)context
{
    Photo *photo = nil;
    
    // ask the database if this photo already exists
    NSString *unique = photoDictionary[FLICKR_PHOTO_ID];
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Photo"];
    request.predicate = [NSPredicate predicateWithFormat:@"unique = %@", unique];
    
    // execute the fetch request
    NSError *error;
    NSArray *matches = [context executeFetchRequest:request error:&error];
    
    // matches is nil or an error occured of there is more than one match
    if (!matches || error || ([matches count] > 1)) {
        // handle error
    } else if ([matches count]) {
        // photo exists, get it
        photo = [matches firstObject];
    } else {
        // photo does not exist, create it
        photo = [NSEntityDescription insertNewObjectForEntityForName:@"Photo"
                                              inManagedObjectContext:context];
        
        // initialize all the attributes
        photo.unique = unique;
        photo.title = [photoDictionary valueForKeyPath:FLICKR_PHOTO_TITLE];
        photo.subtitle = [photoDictionary valueForKeyPath:FLICKR_PHOTO_DESCRIPTION];
        photo.imageURL = [[FlickrFetcher URLforPhoto:photoDictionary
                                              format:FlickrPhotoFormatLarge] absoluteString];
        
        NSString *photographerName = [photoDictionary valueForKeyPath:FLICKR_PHOTO_OWNER];
        photo.whoTook = [Photographer photographerWithName:photographerName
                                    inManagedObjectContext:context];
    }
    
    return photo;
}




+ (void)loadPhotosFromFlickrArray:(NSArray *)photos // of Flickr NSDictionary
         intoManagedObjectContext:(NSManagedObjectContext *)context
{
    for (NSDictionary *photo in photos) {
        [self photoWithFlickrInfo:photo inManagedObjectContext:context];
    }
}

@end
